@@ -1,6 +1,7 @@
 // Iostreams base classes -*- C++ -*-
 
-// Copyright (C) 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2005, 2009
+// Copyright (C) 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2005,
+// 2009, 2010
 // Free Software Foundation, Inc.
 //
 // This file is part of the GNU ISO C++ Library.  This library is free
@@ -33,19 +34,19 @@
 _GLIBCXX_BEGIN_NAMESPACE(std)
 
   // Called only by basic_ios<>::init.
-  void 
-  ios_base::_M_init()   
+  void
+  ios_base::_M_init() throw()
   {
     // NB: May be called more than once
     _M_precision = 6;
     _M_width = 0;
     _M_flags = skipws | dec;
     _M_ios_locale = locale();
-  }  
-  
+  }
+
   // 27.4.2.3  ios_base locale functions
   locale
-  ios_base::imbue(const locale& __loc)
+  ios_base::imbue(const locale& __loc) throw()
   {
     locale __old = _M_ios_locale;
     _M_ios_locale = __loc;
