@@ -1,11 +1,11 @@
 // 1999-08-11 bkoz
 
-// Copyright (C) 1999, 2000, 2001, 2002, 2003 Free Software Foundation
+// Copyright (C) 1999, 2000, 2001, 2002, 2003, 2009 Free Software Foundation
 //
 // This file is part of the GNU ISO C++ Library.  This library is free
 // software; you can redistribute it and/or modify it under the
 // terms of the GNU General Public License as published by the
-// Free Software Foundation; either version 2, or (at your option)
+// Free Software Foundation; either version 3, or (at your option)
 // any later version.
 
 // This library is distributed in the hope that it will be useful,
@@ -14,32 +14,25 @@
 // GNU General Public License for more details.
 
 // You should have received a copy of the GNU General Public License along
-// with this library; see the file COPYING.  If not, write to the Free
-// Software Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307,
-// USA.
-
-// The ARM simulator does not provide support for "fstat", which
-// causes "in_avail" to return an incorrect value.
-// { dg-do run { xfail arm-none-elf } }
+// with this library; see the file COPYING3.  If not see
+// <http://www.gnu.org/licenses/>.
 
 // 27.6.1.3 unformatted input functions
-// @require@ %-*.tst %-*.txt
-// @diff@ %-*.tst %-*.txt
 
 #include <istream>
-#include <fstream>
+#include <sstream>
 #include <testsuite_hooks.h>
 
 // libstdc++/6746   
-void test13()
+void test12()
 {
   using namespace std;
   bool test __attribute__((unused)) = true;
   streamsize sum = 0;
-  ifstream ifs("istream_unformatted-1.tst");
+  istringstream iss("shamma shamma");
       
   // test01
-  size_t i = ifs.rdbuf()->in_avail();
+  size_t i = iss.rdbuf()->in_avail();
   VERIFY( i != 0 );
     
   // test02
@@ -47,16 +40,16 @@ void test13()
   do
     {
       char buf[1024];
-      extracted = ifs.readsome(buf, sizeof buf);
+      extracted = iss.readsome(buf, sizeof buf);
       sum += extracted;
     }
-  while (ifs.good() && extracted);
+  while (iss.good() && extracted);
   VERIFY( sum != 0 );  
 }
- 
+
 int 
 main()
 {
-  test13();
+  test12();
   return 0;
 }
